@@ -37,7 +37,7 @@ DEFAULT_LAYOUT = "layouts/default.html"
 # -------------------------------- SSG.py -------------------------------------
 # ==============================================================================
 
-
+HTML_REDIRECT_TEMPLATE = """<meta http-equiv="Refresh" content="0; url='{url}'" />"""
 
 def print_info(message):
     print("- %s" % message)
@@ -47,19 +47,20 @@ class Generator(object):
     default_page_meta = {
         "title": "",            # The title of the page
         "markup": None,         # The markup to use. ie: md | html (default)
-        # The pretty url new name of the file. A file with the same name will be created
-        "slug": None,
+        "slug": None,           # The pretty url new name of the file. A file with the same name will be created
+        "permalink": None,      # to create a full unique name. It can have slashes etc. ie: /some/page/deep/pages
         "url": "",              # This will be added when processed. Should never be modified
         "description": "",      # Page description
         # By default, all url will be pretty (search engine friendly) Set to False to keep the .html
         "pretty_url": True,
         "meta": {},
+        "redirect": None,       # To allow an html redirect - the page to redirect to. 
         "layout": None,         # The layout for the page
         "template": None,       # The page template.
         "sfc": True,            # For single file component
         "__assets": {           # Contains all assets generated
-            "js": [],      # List of all js url in the page
-            "css": []   # List of all CSS url in the page
+            "js": [],           # List of all js url in the page
+            "css": []           # List of all CSS url in the page
         }
     }
     tpl_env = None
@@ -227,18 +228,26 @@ class Generator(object):
         else:
             return self._get_page_meta_data(s[0])
         
-    def _get_page_url(self, page):
+    def _get_page_url(self, page, full_url=False):
         """ Get the url of a  page """
         anchor = ""
         if "#" in page:
             page, anchor = page.split("#")
             anchor = "#" + anchor
         meta = self._get_page_meta_data(page)
-        return self._make_url(meta.get("url"))
-
-    def _make_url(self, url):
-        return self.base_url.rstrip("/") + "/" + url.lstrip("/")
+        return self._make_url(meta.get("url"), full_url=full_url)
 
+    def _make_url(self, url, full_url=False):
+        """
+        TODO: FIX relative path where inside of subfolder
+        """
+        if full_url:
+            return self.base_url.rstrip("/") + "/" + url.lstrip("/")
+        url = "./" + url.strip("/")
+        if url == "./":
+            url = "../"
+        return url
+        
     def _get_page_static_bundle_url(self, page, bundle_name):
         _page = lib.dictdot(page)
         if url := _page.get("__assets.%s" % bundle_name):
@@ -254,13 +263,21 @@ class Generator(object):
     def _get_dest_file_and_url(self, filepath, page_meta={}):
         """ Return tuple of the file destination and url """
 
-        filename = filepath.split("/")[-1]
-        filepath_base = filepath.replace(filename, "").rstrip("/")
-        slug = page_meta.get("slug")
-        fname = slugify(slug) if slug else filename \
-            .replace(".html", "") \
-            .replace(".md", "")
+        slugname = None
+        # permalink 
+        if permalink := page_meta.get("permalink"):
+            filepath = permalink
+            if not filepath.endswith((".html", ".md")):
+                filepath += ".html"
 
+        elif slug := page_meta.get("slug"):
+            slugname = slugify(slug)
+
+        filename = filepath.split("/")[-1]
+        slugname = slugname or filename
+        filepath_base = filepath.replace(filename, "").rstrip("/")
+        fname = slugname.replace(".html", "").replace(".md", "")
+            
         if page_meta.get("pretty_url") is False:
             dest_file = os.path.join(filepath_base, "%s.html" % fname)
         else:
@@ -479,6 +496,7 @@ class Generator(object):
         :return:
         """
 
+        _context = lib.dictdot(context)
         build_dir = build_dir.rstrip("/")
         filepath = filepath.lstrip("/").rstrip("/")
         if not filepath.endswith(".html"):
@@ -490,7 +508,7 @@ class Generator(object):
         if not os.path.isdir(dest_dir):
             os.makedirs(dest_dir)
 
-        _context = lib.dictdot(context)
+        
         if "page" not in _context:
             _context["page"] = self.default_page_meta.copy()
         if "url" not in _context["page"]:
@@ -499,6 +517,15 @@ class Generator(object):
         self._reset_page_context_assets(_context)
         _context["this"] = _context["page"]
 
+        #! redirect
+        if redirect := _context.get("page.redirect"):
+            url = redirect if lib.is_external_url(redirect) else self._get_page_url(redirect, full_url=True)
+            content = HTML_REDIRECT_TEMPLATE.format(url=url)
+            with open(dest_file, "w") as fw:
+                fw.write(content)
+            return
+
+
         if template:
             if template not in self._templates:
                 self._templates[template] = self.tpl_env.get_template(template)
@@ -594,7 +621,6 @@ class Generator(object):
         if (self.print_info):
             print_info('creating page: %s...' % filepath)
         with open(dest_file, "w") as fw:
-            
             fw.write(tpl.render(**_context))
 
     def build(self, print_info=False):
